from marshmallow import pre_load, fields
from marshmallow_sqlalchemy import SQLAlchemySchema
from backend.utils.misc import NotProvided


class BaseSchema(SQLAlchemySchema):

    SKIP_VALUES = [NotProvided]

    def set_nested_session(self, data):
        """Allow nested schemas to use the parent schema's session. This is a
        longstanding bug with marshmallow-sqlalchemy.

        https://github.com/marshmallow-code/marshmallow-sqlalchemy/issues/67
        https://github.com/marshmallow-code/marshmallow/issues/658#issuecomment-328369199
        """
        nested_fields = {k: v for k, v in self.fields.items() if type(v) == fields.Nested}
        for field in nested_fields.values():
            field.schema.session = self.session

    @pre_load
    def remove_skip_values(self, data, **kwargs):
        self.set_nested_session(data)
        if data is None:
            return {}
        return {key: value for key, value in data.items() if value not in self.SKIP_VALUES}
