import os
from flask import Flask
from multiprocessing import Process
import warnings
from marshmallow.warnings import RemovedInMarshmallow4Warning

warnings.filterwarnings("ignore", category=DeprecationWarning) 
warnings.filterwarnings("ignore", category=RemovedInMarshmallow4Warning)

from backend.utils.launch_utils import set_db_prefix_env, run_migration_upgrade_func, run_create_app

app: Flask = app  # type: ignore

set_db_prefix_env()

from backend.models import db


def set_flask_env():
    os.environ["FLASK_ENV"] = "dss"


def run_migration_upgrade():
    p = Process(
        target=run_migration_upgrade_func
    )  # We have to do this in another process or webapp somehow won't start
    p.start()
    p.join(timeout=10)


def create_app(run_migration=True):
    """
    Mandatory entry point for Flask
    """
    set_flask_env()

    if run_migration:
        run_migration_upgrade()

    run_create_app(app, db)
    return app


create_app()
